@@ -21,7 +21,7 @@
 // Copyright (C) 2008 Hugo Mercier <hmercier31@gmail.com>
 // Copyright (C) 2008 Pino Toscano <pino@kde.org>
 // Copyright (C) 2008 Tomas Are Haavet <tomasare@gmail.com>
-// Copyright (C) 2009-2011, 2013, 2016-2024 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2009-2011, 2013, 2016-2025 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2012, 2013 Fabio D'Urso <fabiodurso@hotmail.it>
 // Copyright (C) 2012, 2015 Tobias Koenig <tokoe@kdab.com>
 // Copyright (C) 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
@@ -46,6 +46,7 @@
 // Copyright (C) 2024 Carsten Emde <ce@ceek.de>
 // Copyright (C) 2024 Lucas Baudin <lucas.baudin@ensae.fr>
 // Copyright (C) 2024, 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2025 Juraj Šarinay <juraj@sarinay.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -55,8 +56,9 @@
 #ifndef ANNOT_H
 #define ANNOT_H
 
-#include <memory>
+#include <array>
 #include <atomic>
+#include <memory>
 #include <mutex>
 #include <vector>
 
@@ -376,12 +378,12 @@ public:
     void adjustColor(int adjust);
 
     AnnotColorSpace getSpace() const { return (AnnotColorSpace)length; }
-    const double *getValues() const { return values; }
+    const std::array<double, 4> &getValues() const { return values; }
 
     Object writeToObject(XRef *xref) const;
 
 private:
-    double values[4];
+    std::array<double, 4> values;
     int length;
 };
 
@@ -553,7 +555,7 @@ public:
 
     // The following functions operate on coords relative to [origX origY]
     void extendTo(double x, double y);
-    void getBBoxRect(double bbox[4]) const;
+    std::array<double, 4> getBBoxRect() const;
 
     // Get boundaries in page coordinates
     double getPageXMin() const;
@@ -624,13 +626,13 @@ private:
                              XRef *xref, Dict *resourcesDict);
     bool drawSignatureFieldText(const FormFieldSignature *field, const Form *form, const GfxResources *resources, const GooString *da, const AnnotBorder *border, const AnnotAppearanceCharacs *appearCharacs, const PDFRectangle *rect,
                                 XRef *xref, Dict *resourcesDict);
-    void drawSignatureFieldText(const GooString &text, const Form *form, const DefaultAppearance &da, const AnnotBorder *border, const PDFRectangle *rect, XRef *xref, Dict *resourcesDict, double leftMargin, bool centerVertically,
+    void drawSignatureFieldText(const std::string &text, const Form *form, const DefaultAppearance &da, const AnnotBorder *border, const PDFRectangle *rect, XRef *xref, Dict *resourcesDict, double leftMargin, bool centerVertically,
                                 bool centerHorizontally);
     bool drawText(const GooString *text, const Form *form, const GooString *da, const GfxResources *resources, const AnnotBorder *border, const AnnotAppearanceCharacs *appearCharacs, const PDFRectangle *rect,
                   const VariableTextQuadding quadding, XRef *xref, Dict *resourcesDict, const int flags = NoDrawTextFlags, const int nCombs = 0);
     void drawArrowPath(double x, double y, const Matrix &m, int orientation = 1);
 
-    GooString *appearBuf;
+    std::unique_ptr<GooString> appearBuf;
 };
 
 //------------------------------------------------------------------------
@@ -721,9 +723,6 @@ public:
 
     static double calculateFontSize(const Form *form, const GfxFont *font, const GooString *text, const double wMax, const double hMax, const bool forceZapfDingbats = {});
 
-    void incRefCnt();
-    void decRefCnt();
-
     virtual void draw(Gfx *gfx, bool printing);
     // Get the resource dict of the appearance stream
     virtual Object getAppearanceResDict();
@@ -780,6 +779,8 @@ public:
     // If newFontNeeded is not null, it will contain whether the given font has glyphs to represent the needed text
     static void layoutText(const GooString *text, GooString *outBuf, int *i, const GfxFont &font, double *width, double widthLimit, int *charCount, bool noReencode, bool *newFontNeeded = nullptr);
 
+    virtual ~Annot();
+
 private:
     void readArrayNum(Object *pdfArray, int key, double *value);
     // write vStr[i:j[ in appearBuf
@@ -788,10 +789,9 @@ private:
     void setPage(int pageIndex, bool updateP); // Called by Page::addAnnot and Annots ctor
 
 protected:
-    virtual ~Annot();
     virtual void removeReferencedObjects(); // Called by Page::removeAnnot
-    Object createForm(const GooString *appearBuf, const double *bbox, bool transparencyGroup, Dict *resDict);
-    Object createForm(const GooString *appearBuf, const double *bbox, bool transparencyGroup, Object &&resDictObject); // overload to support incRef/decRef
+    Object createForm(const GooString *appearBuf, const std::array<double, 4> &bbox, bool transparencyGroup, Dict *resDict);
+    Object createForm(const GooString *appearBuf, const std::array<double, 4> &bbox, bool transparencyGroup, Object &&resDictObject); // overload to support incRef/decRef
     Dict *createResourcesDict(const char *formName, Object &&formStream, const char *stateName, double opacity, const char *blendMode);
     bool isVisible(bool printing);
     int getRotation() const;
@@ -805,8 +805,6 @@ protected:
 
     Object annotObj;
 
-    std::atomic_int refCnt;
-
     // required data
     AnnotSubtype type; // Annotation type
     std::unique_ptr<PDFRectangle> rect; // Rect
@@ -835,6 +833,7 @@ protected:
     mutable std::recursive_mutex mutex;
 
     bool hasBeenUpdated = false;
+    Ref updatedAppearanceStream = Ref::INVALID(); // {-1,-1} if updateAppearanceStream has never been called
 };
 
 //------------------------------------------------------------------------
@@ -879,7 +878,7 @@ public:
 
     // getters
     const GooString *getLabel() const { return label.get(); }
-    AnnotPopup *getPopup() const { return popup.get(); }
+    std::shared_ptr<AnnotPopup> getPopup() const { return popup; }
     double getOpacity() const { return opacity; }
     // getRC
     const GooString *getDate() const { return date.get(); }
@@ -890,7 +889,7 @@ public:
     AnnotExternalDataType getExData() const { return exData; }
 
     // The annotation takes the ownership of new_popup
-    void setPopup(std::unique_ptr<AnnotPopup> &&new_popup);
+    void setPopup(std::shared_ptr<AnnotPopup> new_popup);
     void setLabel(std::unique_ptr<GooString> &&new_label);
     void setOpacity(double opacityA);
     void setDate(std::unique_ptr<GooString> new_date);
@@ -899,7 +898,7 @@ protected:
     void removeReferencedObjects() override;
 
     std::unique_ptr<GooString> label; // T            (Default author)
-    std::unique_ptr<AnnotPopup> popup; // Popup
+    std::shared_ptr<AnnotPopup> popup; // Popup
     double opacity; // CA           (Default 1.0)
     // RC
     std::unique_ptr<GooString> date; // CreationDate
@@ -944,17 +943,17 @@ public:
 
     // getters
     bool getOpen() const { return open; }
-    const GooString *getIcon() const { return icon.get(); }
+    const std::string &getIcon() const { return icon; }
     AnnotTextState getState() const { return state; }
 
     void setOpen(bool openA);
-    void setIcon(GooString *new_icon);
+    void setIcon(const std::string &new_icon);
 
 private:
     void initialize(PDFDoc *docA, Dict *dict);
 
     bool open; // Open       (Default false)
-    std::unique_ptr<GooString> icon; // Name       (Default Note)
+    std::string icon; // Name       (Default Note)
     AnnotTextState state; // State      (Default Umarked if
                           //             StateModel Marked
                           //             None if StareModel Review)
@@ -1074,7 +1073,7 @@ public:
     void setDefaultAppearance(const DefaultAppearance &da);
     void setQuadding(VariableTextQuadding new_quadding);
     void setStyleString(GooString *new_string);
-    void setCalloutLine(AnnotCalloutLine *line);
+    void setCalloutLine(std::unique_ptr<AnnotCalloutLine> &&line);
     void setIntent(AnnotFreeTextIntent new_intent);
 
     // getters
@@ -1228,14 +1227,12 @@ public:
 
     void draw(Gfx *gfx, bool printing) override;
 
-    void setIcon(GooString *new_icon);
+    void setIcon(const std::string &new_icon);
 
-    void setCustomImage(AnnotStampImageHelper *stampImageHelperA);
-
-    void clearCustomImage();
+    void setCustomImage(std::unique_ptr<AnnotStampImageHelper> &&stampImageHelperA);
 
     // getters
-    const GooString *getIcon() const { return icon.get(); }
+    const std::string &getIcon() const { return icon; }
 
     Object getAppearanceResDict() override;
 
@@ -1245,9 +1242,8 @@ private:
     void generateStampCustomAppearance();
     void updateAppearanceResDict();
 
-    std::unique_ptr<GooString> icon; // Name       (Default Draft)
-    AnnotStampImageHelper *stampImageHelper;
-    Ref updatedAppearanceStream;
+    std::string icon; // Name       (Default Draft)
+    std::unique_ptr<AnnotStampImageHelper> stampImageHelper;
 };
 
 //------------------------------------------------------------------------
@@ -1465,7 +1461,6 @@ public:
     ~AnnotWidget() override;
 
     void draw(Gfx *gfx, bool printing) override;
-    void invalidateAppearance() override;
 
     void generateFieldAppearance();
     void updateAppearanceStream();
@@ -1494,7 +1489,6 @@ private:
     // inherited  from Annot
     // AnnotBorderBS border;                // BS
     Dict *parent; // Parent
-    Ref updatedAppearanceStream; // {-1,-1} if updateAppearanceStream has never been called
 };
 
 //------------------------------------------------------------------------
@@ -1771,17 +1765,17 @@ public:
     Annots(const Annots &) = delete;
     Annots &operator=(const Annots &) = delete;
 
-    const std::vector<Annot *> &getAnnots() { return annots; }
+    const std::vector<std::shared_ptr<Annot>> &getAnnots() { return annots; }
 
-    void appendAnnot(Annot *annot);
-    bool removeAnnot(Annot *annot);
+    void appendAnnot(std::shared_ptr<Annot> annot);
+    bool removeAnnot(const std::shared_ptr<Annot> &annot);
 
 private:
-    Annot *createAnnot(Object &&dictObject, const Object *obj);
-    Annot *findAnnot(Ref *ref);
+    std::shared_ptr<Annot> createAnnot(Object &&dictObject, const Object *obj);
+    std::shared_ptr<Annot> findAnnot(Ref *ref);
 
     PDFDoc *doc;
-    std::vector<Annot *> annots;
+    std::vector<std::shared_ptr<Annot>> annots;
 };
 
 #endif
